@@ -22,58 +22,56 @@ module Facebook.Auth
 #if __GLASGOW_HASKELL__ <= 784
 import Control.Applicative
 #endif
-import Control.Monad (guard, liftM, mzero)
-import Control.Monad.IO.Class (MonadIO(liftIO))
+import Control.Monad (guard, mzero)
+import Control.Monad.IO.Class
 import Control.Monad.Trans.Maybe (MaybeT(..))
-import Crypto.Classes (constTimeEq)
-import Crypto.Hash.CryptoAPI (SHA256)
-import Crypto.HMAC (hmac', MacKey(..))
-import Data.Aeson ((.:))
-import Data.Aeson.Parser (json')
-import Data.Maybe (fromMaybe)
-import Data.Text (Text)
-import Data.Time (getCurrentTime, addUTCTime, UTCTime)
-import Data.Typeable (Typeable)
-import Data.String (IsString(..))
-
-import qualified UnliftIO.Exception as E
 import qualified Control.Monad.Trans.Resource as R
+import Crypto.Hash.Algorithms (SHA256)
+import Crypto.MAC.HMAC (HMAC(..), hmac)
+import Data.Aeson ((.:))
 import qualified Data.Aeson as AE
+import Data.Aeson.Parser (json')
 import qualified Data.Aeson.Types as AE
 import qualified Data.Attoparsec.ByteString.Char8 as AB
+import Data.ByteArray (ScrubbedBytes, convert)
+import Data.ByteArray.Encoding (Base(..), convertFromBase)
 import qualified Data.ByteString as B
-import qualified Data.ByteString.Base64.URL as Base64URL
 import qualified Data.ByteString.Char8 as B8
 import qualified Data.List as L
-import qualified Data.Serialize as Cereal
+import Data.Maybe (fromMaybe)
+import Data.String (IsString(..))
+import Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as TE
 import qualified Data.Text.Encoding.Error as TE
+import Data.Time (UTCTime, addUTCTime, getCurrentTime)
+import Data.Typeable (Typeable)
 import qualified Network.HTTP.Types as HT
+import qualified UnliftIO.Exception as E
 
-import Facebook.Types
 import Facebook.Base
 import Facebook.Monad
+import Facebook.Types
 
 -- | Get an app access token from Facebook using your
 -- credentials.
 -- Ref: https://developers.facebook.com/docs/facebook-login/manually-build-a-login-flow
-getAppAccessToken
-  :: (R.MonadResource m, R.MonadUnliftIO m, R.MonadThrow m)
+getAppAccessToken ::
+     (R.MonadResource m, R.MonadUnliftIO m, R.MonadThrow m, MonadIO m)
   => FacebookT Auth m AppAccessToken
 getAppAccessToken =
-  runResourceInFb $
-  do creds <- getCreds
-     req <-
-       fbreq "/oauth/access_token" Nothing $
-       tsq creds [("grant_type", "client_credentials")]
-     response <- fbhttp req
-     (token :: AE.Value) <- asJson response
-     case AE.parseMaybe tokenParser token of
-       Just appToken -> return $ AppAccessToken appToken
-       _ ->
-         E.throwIO $
-         FbLibraryException ("Unable to parse: " <> (T.pack $ show token))
+  runResourceInFb $ do
+    creds <- getCreds
+    req <-
+      fbreq "/oauth/access_token" Nothing $
+      tsq creds [("grant_type", "client_credentials")]
+    response <- fbhttp req
+    (token :: AE.Value) <- asJson response
+    case AE.parseMaybe tokenParser token of
+      Just appToken -> return $ AppAccessToken appToken
+      _ ->
+        E.throwIO $
+        FbLibraryException ("Unable to parse: " <> (T.pack $ show token))
   where
     tokenParser :: AE.Value -> AE.Parser AccessTokenData
     tokenParser val =
@@ -88,29 +86,31 @@ getAppAccessToken =
 -- Facebook URL you should redirect you user to.  Facebook will
 -- authenticate the user, authorize your app and then redirect
 -- the user back into the provider 'RedirectUrl'.
-getUserAccessTokenStep1
-  :: Monad m
-  => RedirectUrl -> [Permission] -> FacebookT Auth m Text
+getUserAccessTokenStep1 ::
+     (Monad m, MonadIO m)
+  => RedirectUrl
+  -> [Permission]
+  -> FacebookT Auth m Text
 getUserAccessTokenStep1 redirectUrl perms = do
   creds <- getCreds
-  withTier $
-    \tier ->
-       let urlBase =
-             case tier of
-               Production ->
-                 "https://www.facebook.com/" <> apiVersion <>
-                 "/dialog/oauth?client_id="
-               Beta ->
-                 "https://www.beta.facebook.com/" <> apiVersion <>
-                 "/dialog/oauth?client_id="
-       in T.concat $
-          urlBase :
-          appId creds :
-          "&redirect_uri=" :
-          redirectUrl :
-          (case perms of
-             [] -> []
-             _ -> "&scope=" : L.intersperse "," (map unPermission perms))
+  apiVersion <- getApiVersion
+  withTier $ \tier ->
+    let urlBase =
+          case tier of
+            Production ->
+              "https://www.facebook.com/" <> apiVersion <>
+              "/dialog/oauth?client_id="
+            Beta ->
+              "https://www.beta.facebook.com/" <> apiVersion <>
+              "/dialog/oauth?client_id="
+     in T.concat $
+        urlBase :
+        appId creds :
+        "&redirect_uri=" :
+        redirectUrl :
+        (case perms of
+           [] -> []
+           _ -> "&scope=" : L.intersperse "," (map unPermission perms))
 
 -- | The second step to get an user access token.  If the user is
 -- successfully authenticate and they authorize your application,
@@ -119,8 +119,8 @@ getUserAccessTokenStep1 redirectUrl perms = do
 -- request query parameters passed to your 'RedirectUrl' and give
 -- to this function that will complete the user authentication
 -- flow and give you an @'UserAccessToken'@.
-getUserAccessTokenStep2
-  :: (R.MonadResource m, R.MonadUnliftIO m, R.MonadThrow m)
+getUserAccessTokenStep2 ::
+     (R.MonadResource m, R.MonadUnliftIO m, R.MonadThrow m, MonadIO m)
   => RedirectUrl -- ^ Should be exactly the same
      -- as in 'getUserAccessTokenStep1'.
   -> [Argument] -- ^ Query parameters.
@@ -130,23 +130,24 @@ getUserAccessTokenStep2 redirectUrl query =
     [code@("code", _)] ->
       runResourceInFb $
       -- Get the access token data through Facebook's OAuth.
-      do now <- liftIO getCurrentTime
-         creds <- getCreds
-         req <-
-           fbreq "/oauth/access_token" Nothing $
-           tsq creds [code, ("redirect_uri", TE.encodeUtf8 redirectUrl)]
-         response <- fbhttp req
-         (userToken :: AE.Value) <- asJson response
-         let (token, expire) = userAccessTokenParser now userToken
+       do
+        now <- liftIO getCurrentTime
+        creds <- getCreds
+        req <-
+          fbreq "/oauth/access_token" Nothing $
+          tsq creds [code, ("redirect_uri", TE.encodeUtf8 redirectUrl)]
+        response <- fbhttp req
+        (userToken :: AE.Value) <- asJson response
+        let (token, expire) = userAccessTokenParser now userToken
          -- Get user's ID throught Facebook's graph.
-         userResponse <-
-           fbhttp =<<
-           fbreq
-             "/me"
-             (Just (UserAccessToken "invalid id" token expire))
-             [("fields", "id")]
-         (userId :: UserId) <- asJson userResponse
-         return $ UserAccessToken userId token expire
+        userResponse <-
+          fbhttp =<<
+          fbreq
+            "/me"
+            (Just (UserAccessToken "invalid id" token expire))
+            [("fields", "id")]
+        (userId :: UserId) <- asJson userResponse
+        return $ UserAccessToken userId token expire
     _ ->
       let [error_, errorReason, errorDescr] =
             map
@@ -154,13 +155,13 @@ getUserAccessTokenStep2 redirectUrl query =
               ["error", "error_reason", "error_description"]
           errorType = T.concat [t error_, " (", t errorReason, ")"]
           t = TE.decodeUtf8With TE.lenientDecode
-      in E.throwIO $ FacebookException errorType (t errorDescr)
+       in E.throwIO $ FacebookException errorType (t errorDescr)
 
 -- | Attoparsec parser for user access tokens returned by
 -- Facebook as a query string.  Returns an user access token with
 -- a broken 'UserId'.
-userAccessTokenParser
-  :: UTCTime -- ^ 'getCurrentTime'
+userAccessTokenParser ::
+     UTCTime -- ^ 'getCurrentTime'
   -> AE.Value
   -> (AccessTokenData, UTCTime)
 userAccessTokenParser now val =
@@ -191,8 +192,8 @@ userAccessTokenParser now val =
 -- to prevent this bug, we suggest that you use 'isValid' before
 -- redirecting the user to the URL provided by 'getUserLogoutUrl'
 -- since this function doesn't do any validity checks.
-getUserLogoutUrl
-  :: Monad m
+getUserLogoutUrl ::
+     Monad m
   => UserAccessToken
      -- ^ The user's access token.
   -> RedirectUrl
@@ -203,19 +204,18 @@ getUserLogoutUrl
 -- @https:\/\/www.beta.facebook.com\/@ when
 -- using the beta tier).
 getUserLogoutUrl (UserAccessToken _ data_ _) next = do
-  withTier $
-    \tier ->
-       let urlBase =
-             case tier of
-               Production -> "https://www.facebook.com/logout.php?"
-               Beta -> "https://www.beta.facebook.com/logout.php?"
-       in TE.decodeUtf8 $
-          urlBase <>
-          HT.renderQuery
-            False
-            [ ("next", Just (TE.encodeUtf8 next))
-            , ("access_token", Just (TE.encodeUtf8 data_))
-            ]
+  withTier $ \tier ->
+    let urlBase =
+          case tier of
+            Production -> "https://www.facebook.com/logout.php?"
+            Beta -> "https://www.beta.facebook.com/logout.php?"
+     in TE.decodeUtf8 $
+        urlBase <>
+        HT.renderQuery
+          False
+          [ ("next", Just (TE.encodeUtf8 next))
+          , ("access_token", Just (TE.encodeUtf8 data_))
+          ]
 
 -- | URL where the user is redirected to after Facebook
 -- authenticates the user authorizes your application.  This URL
@@ -236,12 +236,14 @@ type RedirectUrl = Text
 -- >
 -- > perms :: [Permission]
 -- > perms = ["user_about_me", "email", "offline_access"]
-newtype Permission = Permission
-  { unPermission :: Text
+newtype Permission =
+  Permission
+    { unPermission :: Text
     -- ^ Retrieves the 'Text' back from a 'Permission'.  Most of
     -- the time you won't need to use this function, but you may
     -- need it if you're a library author.
-  } deriving (Eq, Ord)
+    }
+  deriving (Eq, Ord)
 
 instance Show Permission where
   show = show . unPermission
@@ -250,9 +252,7 @@ instance IsString Permission where
   fromString = Permission . fromString
 
 -- | @True@ if the access token has expired, otherwise @False@.
-hasExpired
-  :: (Functor m, MonadIO m)
-  => AccessToken anyKind -> m Bool
+hasExpired :: (Functor m, MonadIO m) => AccessToken anyKind -> m Bool
 hasExpired token =
   case accessTokenExpires token of
     Nothing -> return False
@@ -263,9 +263,10 @@ hasExpired token =
 -- access token may not be valid as well.  For example, in the
 -- case of an user access token, they may have changed their
 -- password, logged out from Facebook or blocked your app.
-isValid
-  :: (R.MonadResource m, R.MonadUnliftIO m)
-  => AccessToken anyKind -> FacebookT anyAuth m Bool
+isValid ::
+     (R.MonadResource m, R.MonadUnliftIO m)
+  => AccessToken anyKind
+  -> FacebookT anyAuth m Bool
 isValid token = do
   expired <- hasExpired token
   if expired
@@ -283,7 +284,7 @@ isValid token = do
              -- will actually work with user access tokens,
              -- too, but they have another, better way of
              -- being checked.
-         in httpCheck =<< fbreq page (Just token) []
+          in httpCheck =<< fbreq page (Just token) []
 
 -- | Extend the expiration time of an user access token (see
 -- <https://developers.facebook.com/docs/offline-access-deprecation/>,
@@ -296,8 +297,8 @@ isValid token = do
 -- have the same data and expiration time as before, but you
 -- can't assume this).  Note that expired access tokens can't be
 -- extended, only valid tokens.
-extendUserAccessToken
-  :: (R.MonadResource m, R.MonadUnliftIO m, R.MonadThrow m)
+extendUserAccessToken ::
+     (R.MonadResource m, R.MonadUnliftIO m, R.MonadThrow m, MonadIO m)
   => UserAccessToken
   -> FacebookT Auth m (Either FacebookException UserAccessToken)
 extendUserAccessToken token@(UserAccessToken uid data_ _) = do
@@ -307,23 +308,23 @@ extendUserAccessToken token@(UserAccessToken uid data_ _) = do
     else tryToExtend
   where
     tryToExtend =
-      runResourceInFb $
-      do creds <- getCreds
-         req <-
-           fbreq "/oauth/access_token" Nothing $
-           tsq
-             creds
-             [ ("grant_type", "fb_exchange_token")
-             , ("fb_exchange_token", TE.encodeUtf8 data_)
-             ]
-         response <- fbhttp req
-         userToken <- E.try $ asJson response
-         case userToken of
-           Right val -> do
-             now <- liftIO getCurrentTime
-             let (extendedtoken, expire) = userAccessTokenParser now val
-             return $ Right $ UserAccessToken uid extendedtoken expire
-           Left exc -> return (Left exc)
+      runResourceInFb $ do
+        creds <- getCreds
+        req <-
+          fbreq "/oauth/access_token" Nothing $
+          tsq
+            creds
+            [ ("grant_type", "fb_exchange_token")
+            , ("fb_exchange_token", TE.encodeUtf8 data_)
+            ]
+        response <- fbhttp req
+        userToken <- E.try $ asJson response
+        case userToken of
+          Right val -> do
+            now <- liftIO getCurrentTime
+            let (extendedtoken, expire) = userAccessTokenParser now val
+            return $ Right $ UserAccessToken uid extendedtoken expire
+          Left exc -> return (Left exc)
     hasExpiredExc =
       mkExc
         [ "the user access token has already expired, "
@@ -335,38 +336,41 @@ extendUserAccessToken token@(UserAccessToken uid data_ _) = do
 -- (<https://developers.facebook.com/docs/authentication/signed_request/>),
 -- verifies its authencity and integrity using the HMAC and
 -- decodes its JSON object.
-parseSignedRequest
-  :: (AE.FromJSON a, Monad m)
+parseSignedRequest ::
+     (AE.FromJSON a, Monad m, MonadIO m)
   => B8.ByteString -- ^ Encoded Facebook signed request
   -> FacebookT Auth m (Maybe a)
 parseSignedRequest signedRequest =
   runMaybeT $
   -- Split, decode and JSON-parse
-  do let (encodedSignature, encodedUnparsedPayloadWithDot) = B8.break (== '.') signedRequest
-     ('.', encodedUnparsedPayload) <-
-       MaybeT $ return (B8.uncons encodedUnparsedPayloadWithDot)
-     signature <- eitherToMaybeT $ Base64URL.decode $ addBase64Padding encodedSignature
-     unparsedPayload <- eitherToMaybeT $ Base64URL.decode $ addBase64Padding encodedUnparsedPayload
-     payload <- eitherToMaybeT $ AB.parseOnly json' unparsedPayload
+   do
+    let (encodedSignature, encodedUnparsedPayloadWithDot) =
+          B8.break (== '.') signedRequest
+    ('.', encodedUnparsedPayload) <-
+      MaybeT $ return (B8.uncons encodedUnparsedPayloadWithDot)
+    signature <-
+      eitherToMaybeT $
+      convertFromBase Base64 $ addBase64Padding encodedSignature
+    unparsedPayload <-
+      eitherToMaybeT $
+      convertFromBase Base64 $ addBase64Padding encodedUnparsedPayload
+    payload <- eitherToMaybeT $ AB.parseOnly json' unparsedPayload
      -- Verify signature
-     SignedRequestAlgorithm algo <- fromJson payload
-     guard (algo == "HMAC-SHA256")
-     hmacKey <- credsToHmacKey `liftM` lift getCreds
-     let expectedSignature = Cereal.encode $ hmac' hmacKey encodedUnparsedPayload
-     guard (signature `constTimeEq` expectedSignature)
+    SignedRequestAlgorithm algo <- fromJson payload
+    guard (algo == "HMAC-SHA256")
+    creds <- lift getCreds
+    let hmacKey = credsToHmacKey creds
+        expectedSignature = hmac hmacKey encodedUnparsedPayload :: HMAC SHA256
+    guard ((signature :: ScrubbedBytes) == (convert expectedSignature))
      -- Parse user data type
-     fromJson payload
+    fromJson payload
   where
-    eitherToMaybeT
-      :: Monad m
-      => Either a b -> MaybeT m b
+    eitherToMaybeT :: Monad m => Either a b -> MaybeT m b
     eitherToMaybeT = MaybeT . return . either (const Nothing) Just
-    fromJson
-      :: (AE.FromJSON a, Monad m)
-      => AE.Value -> MaybeT m a
+    fromJson :: (AE.FromJSON a, Monad m) => AE.Value -> MaybeT m a
     fromJson = eitherToMaybeT . AE.parseEither AE.parseJSON
-    credsToHmacKey :: Credentials -> MacKey ctx SHA256
-    credsToHmacKey = MacKey . appSecretBS
+    -- credsToHmacKey :: Credentials -> MacKey ctx SHA256
+    credsToHmacKey = appSecretBS
 
 newtype SignedRequestAlgorithm =
   SignedRequestAlgorithm Text
@@ -393,8 +397,8 @@ addBase64Padding bs
     drem = B.length bs `mod` 4
 
 -- | Get detailed information about an access token.
-debugToken
-  :: (R.MonadResource m, R.MonadUnliftIO m, R.MonadThrow m)
+debugToken ::
+     (R.MonadResource m, R.MonadUnliftIO m, R.MonadThrow m)
   => AppAccessToken -- ^ Your app access token.
   -> AccessTokenData -- ^ The access token you want to debug.
   -> FacebookT Auth m DebugToken
@@ -406,32 +410,31 @@ debugToken appToken userTokenData = do
   let muserToken =
         UserAccessToken <$> dtUserId ret <*> return userTokenData <*>
         dtExpiresAt ret
-  return
-    ret
-    { dtAccessToken = muserToken
-    }
+  return ret {dtAccessToken = muserToken}
 
 -- | Helper used in 'debugToken'.  Unfortunately, we can't use 'Pager' here.
-data Undata a = Undata
-  { undata :: a
-  }
+data Undata a =
+  Undata
+    { undata :: a
+    }
 
-instance AE.FromJSON a =>
-         AE.FromJSON (Undata a) where
+instance AE.FromJSON a => AE.FromJSON (Undata a) where
   parseJSON (AE.Object v) = Undata <$> v AE..: "data"
   parseJSON _ = mzero
 
 -- | Detailed information about an access token (cf. 'debugToken').
-data DebugToken = DebugToken
-  { dtAppId :: Maybe Text
-  , dtAppName :: Maybe Text
-  , dtExpiresAt :: Maybe UTCTime
-  , dtIsValid :: Maybe Bool
-  , dtIssuedAt :: Maybe UTCTime
-  , dtScopes :: Maybe [Permission]
-  , dtUserId :: Maybe Id
-  , dtAccessToken :: Maybe UserAccessToken
-  } deriving (Eq, Ord, Show, Typeable)
+data DebugToken =
+  DebugToken
+    { dtAppId :: Maybe Text
+    , dtAppName :: Maybe Text
+    , dtExpiresAt :: Maybe UTCTime
+    , dtIsValid :: Maybe Bool
+    , dtIssuedAt :: Maybe UTCTime
+    , dtScopes :: Maybe [Permission]
+    , dtUserId :: Maybe Id
+    , dtAccessToken :: Maybe UserAccessToken
+    }
+  deriving (Eq, Ord, Show, Typeable)
 
 -- | Note: this instance always sets 'dtAccessToken' to
 -- 'Nothing', but 'debugToken' will update this field before
