@@ -1,5 +1,6 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
@@ -60,21 +61,25 @@ import qualified UnliftIO.Exception as E
 -- Facebook.  The phantom type @auth@ may be either 'Auth' (you
 -- have supplied your 'Credentials') or 'NoAuth' (you have not
 -- supplied any 'Credentials').
-newtype FacebookT auth m a =
-  F
-    { unF :: ReaderT FbData m a -- FbData -> m a
-    }
-  deriving ( Functor
-           , Applicative
-           , Alternative
-           , Monad
-           , MonadFix
-           , MonadPlus
-           , MonadIO
-           , MonadTrans
-           , R.MonadThrow
-           , MonadFail
-           )
+newtype FacebookT auth m a = F
+  { unF :: ReaderT FbData m a -- FbData -> m a
+  } deriving ( Functor
+             , Applicative
+             , Alternative
+             , Monad
+             , MonadFix
+             , MonadPlus
+             , MonadIO
+             , MonadTrans
+             , R.MonadThrow
+             , MonadFail
+             )
+
+instance (MonadUnliftIO m) => MonadUnliftIO (FacebookT auth m) where
+  withRunInIO inner =
+    F $
+    ReaderT $ \r ->
+      withRunInIO $ \run -> inner (\fbT -> run $ (flip runReaderT) r (unF fbT))
 
 deriving instance
          (R.MonadResource m, MonadBase IO m) =>
@@ -83,20 +88,6 @@ deriving instance
 instance MonadBase b m => MonadBase b (FacebookT auth m) where
   liftBase = lift . liftBase
 
--- askUnliftIO = ReaderT $ \r ->
---                 withUnliftIO $ \u ->
--- return (UnliftIO (unliftIO u . flip runReaderT r))
-instance (MonadIO m, MonadUnliftIO m) => MonadUnliftIO (FacebookT auth m) where
-  askUnliftIO :: FacebookT auth m (UnliftIO (FacebookT auth m))
-  askUnliftIO =
-    F
-      (ReaderT $ \(r :: FbData) ->
-         withUnliftIO $ \(u :: UnliftIO m) ->
-           return
-             (UnliftIO
-                (\(x :: FacebookT auth m a) ->
-                   (unliftIO u ((flip runReaderT r) (unF x))))))
-
 -- | Since @fb-0.14.8@.
 instance MonadLogger m => MonadLogger (FacebookT auth m) where
   monadLoggerLog loc src lvl msg = lift (monadLoggerLog loc src lvl msg)
@@ -113,14 +104,12 @@ data NoAuth
   deriving (Typeable)
 
 -- | Internal data kept inside 'FacebookT'.
-data FbData =
-  FbData
-    { fbdCreds :: Maybe Credentials
-    , fbdManager :: !H.Manager
-    , fbdTier :: !FbTier
-    , fbdApiVersion :: IORef ApiVersion
-    }
-  deriving (Typeable)
+data FbData = FbData
+  { fbdCreds :: Maybe Credentials
+  , fbdManager :: !H.Manager
+  , fbdTier :: !FbTier
+  , fbdApiVersion :: IORef ApiVersion
+  } deriving (Typeable)
 
 -- | Which Facebook tier should be used (see
 -- <https://developers.facebook.com/support/beta-tier/>).
